#include "entidad.h"
#include "mapa.h"
#include "hero.h"
#include "enemigo.h"
#include "tablas.h"

// Estado de Sprites
const TFrame g_frames[HERO_FRAMES] = {
      { M_derecha, g_hero_00 }   ,  { M_derecha, g_hero_01 }
   ,  { M_derecha, g_hero_02 }   ,  { M_derecha, g_hero_03 }
   ,  { M_derecha, g_hero_04 }   ,  { M_derecha, g_hero_05 }
   ,  { M_derecha, g_hero_06 }   ,  { M_derecha, g_hero_07 }
   ,  { M_derecha, g_hero_08 }   ,  { M_derecha, g_hero_09 }
   ,  { M_derecha, g_hero_10 }   ,  { M_derecha, g_hero_11 }
   ,  { M_derecha, g_hero_12 }   ,  { M_derecha, g_hero_13 }
   ,  { M_derecha, g_hero_14 }   ,  { M_derecha, g_hero_15 }
};

// Animaciones

TFrame* const anim_andar[ANDAR_FRAMES] = { 
   &g_frames[1], &g_frames[2], &g_frames[3], &g_frames[4], &g_frames[5]
};
TFrame* const anim_saltar[SALTAR_FRAMES] = { 
   &g_frames[8], &g_frames[9], &g_frames[10], &g_frames[11]
};
TFrame* const anim_caer[CAER_FRAMES] = { 
   &g_frames[14], &g_frames[15]
};

const CPCT_2BITARRAY(g_tablaSalto, PASOS_SALTO) = {
     CPCT_ENCODE2BITS(3, 3, 3, 3) 
   , CPCT_ENCODE2BITS(3, 3, 3, 3)
   , CPCT_ENCODE2BITS(3, 3, 2, 2)
   , CPCT_ENCODE2BITS(2, 2, 2, 2)
   , CPCT_ENCODE2BITS(2, 1, 1, 0)
};

const TEntidad template = {
   { {74, 74, 74}, {173, 173, 173}, 3 },
   ANCHO_HERO, ALTO_HERO,
   NO,
   ST_quieto,
   &g_frames[0],
   0,
   M_derecha,
   2,
   0,
   1,
   {'0','0','\0'},
   {'3','\0'},
   1,
   1,
   0
};

TEntidad entidad;

// Máscara de transparencia
cpctm_createTransparentMaskTable(g_tablatrans, 0x100, M0, 0);

u8 compruebaTeclas(const cpct_keyID* k, u8 numk) {
   u8 i;
   cpct_scanKeyboard_if();
   if (cpct_isAnyKeyPressed()) {
     for(i=0; i < numk; i++, k++) {
         if (cpct_isKeyPressed(*k))
            return i;
      }
   }
   return -1;
}

void inicializarEntidad(char u) {
	cpct_memcpy((void*)&entidad, &template, sizeof(TEntidad));
   entidad.nvida[0]=u;
}

void asignarFrame(TEntidad* e, TFrame **animacion) {
   e->frame = animacion[e->nframe / ANIM_PAUSA];
}

void girarFrame(TEntidad* e) {
   TFrame* f = e->frame;
   if (f->mira != e->mira) {
      cpct_hflipSpriteM0(G_HERO_00_W, G_HERO_00_H, f->sprite);
      f->mira = e->mira;
   }
}

void seleccionarSpriteEntidad(TEntidad* e) {
   TFrame* frames;
   TFrame** anim_and;
   TFrame** anim_sal;
   TFrame** anim_cae;
   if(e->nbanana[0]=='e')
   {
      frames=g_eframes;
      anim_and=anim_eandar;
      anim_sal=anim_esaltar;
      anim_cae=anim_ecaer;
   }
   else{
      frames=g_frames;
      anim_and=anim_andar;
      anim_sal=anim_saltar;
      anim_cae=anim_caer;
   }

   switch(e->estado) {
      case ST_quieto:      { e->frame = &frames[0];       break; }
      case ST_andando:     { asignarFrame(e, anim_and);   break; }
      case ST_prepSalto:   { asignarFrame(e, anim_sal);  break; }
      case ST_saltando:    { e->frame = &frames[12];      break; }
      case ST_cayendo:     { e->frame = &frames[13];      break; }
      case ST_aterrizando: { asignarFrame(e, anim_cae);    break; }
   }
   girarFrame(e);
}

void dibujarEntidad(TEntidad* e) {
	u8* pvmem= cpct_getScreenPtr(g_backBuffer,e->pos.x[0],e->pos.y[0]);
	cpct_drawSpriteMaskedAlignedTable (e->frame->sprite,pvmem,e->w,e->h, g_tablatrans);
}

void dibujarEntidad_initial(TEntidad* e) {
   u8* pvmem= cpct_getScreenPtr(g_screenMem,e->pos.x[0],e->pos.y[0]);
   cpct_drawSpriteMaskedAlignedTable (e->frame->sprite,pvmem,e->w,e->h, g_tablatrans);
}

void borrarEntidad(TEntidad* e) {
   u8 w = 3 + (e->pos.x[2] & 1);
   u8 h = 5 + (e->pos.y[2] & 3 ? 1 : 0);
   u8* pvmem= cpct_getScreenPtr(g_backBuffer,0,40);
   cpct_etm_drawTileBox2x4(e->pos.x[2] / 2, (e->pos.y[2]-ORIGEN_MAPA_Y) / 4, w, h, 40, pvmem, mapas[e->pos.num_mapa]);
}

void copy3(u8 *p) {
   *p = *(p-1); --p; *p = *(p-1);
}

void entityDrawnUpdate(TEntidad* e) {
   copy3(e->pos.x + 2);
   copy3(e->pos.y + 2);
   --e->draw;
}

void desplazarEntidad(TEntidad* e, u8 i, u8 x, u8 y) {
   e->pos.x[i] = x;
	e->pos.y[i] = y;
	e->mover = SI;
}

void ajustarAlSuelo(TEntidad* e) {
   e->pos.y[0] = (e->pos.y[0] & 0b11111100)+1;
}

void moverDerecha(TEntidad* e) {
	if(e->pos.x[0] + G_HERO_00_W < ANCHO_PANTALLA) {
		if(!comprobarPared(e,SI)){
			desplazarEntidad(e, 0, ++e->pos.x[0], e->pos.y[0]);
			e->mira = M_derecha;
		}
	}else if(e->nbanana[0]!='e')
   {
      avanzarMapa();    
   }
}

void moverIzquierda(TEntidad* e) {
	if(e->pos.x[0] > 0){
		if(!comprobarPared(e,NO)){
			desplazarEntidad(e, 0, --e->pos.x[0], e->pos.y[0]);
			e->mira = M_izquierda;
		}
	}else if(e->nbanana[0]!='e'){
      retrocederMapa();
   }
}

void cambiarEstado(TEntidad* e, u8 estado, u8 mira) {

   if(estado == ST_quieto) {
      e->estado = ST_quieto;
   } else if(estado == ST_andando) {
      e->nframe = 0;
      e->estado = ST_andando;
      e->mira   = mira;
   } else if(estado == ST_prepSalto) {
      e->nframe = 0;
      e->estado = ST_prepSalto;
   } else if(estado == ST_saltando) {
      e->estado = ST_saltando;
      e->salto  = 0;
   } else if(estado == ST_cayendo) {
      e->estado = ST_cayendo;
      e->salto  = PASOS_SALTO - 3;
   } else {
      e->nframe = 0;
      e->estado = ST_aterrizando;
   }

	e->mover  = SI;
   e->draw = 2;
}

void aterrizando(TEntidad* e) {
   if(++e->nframe == CAER_FRAMES*ANIM_PAUSA)
      e->draw = 2;
      
      cambiarEstado(e, ST_quieto, 0);

}

void descender(TEntidad* e) {
	desplazarEntidad(e, 0, e->pos.x[0], (e->pos.y[0]+cpct_get2Bits(g_tablaSalto, e->salto)));
	if (e->salto > 1){
		--e->salto;
	}
}

void cayendo(TEntidad* e) {
   u8 k;
   if(e->nbanana[0]!='e') {
      static const cpct_keyID keys[2] = {Key_A, Key_D};
      k = compruebaTeclas(keys, 2);
   } else {
      k=-1;
   }
   if(k == 1) {
         moverDerecha(e);
      } else if (k == 0) {
         moverIzquierda(e);
      }

   descender(e);

   if (sobreSuelo(e)) {
       if(e->nbanana[0]=='e' && comprobarMovimientoEnemigo(e)==2)
         {
            e->forzarAndar=1;
         }
      ajustarAlSuelo(e);
       e->draw = 2;
      cambiarEstado(e, ST_aterrizando, 0);
   }

   if(bordeInferior(e) && (e->pos.num_mapa == 0 || e->pos.num_mapa == 1)&& (e->nbanana[0]!='e')) {
      bajarMapa();
   }

   else if(bordeInferior(e)) {
      muerteCaida(e);  
   }
}

void quieto(TEntidad* e) {
   u8 k;
   if(e->nbanana[0]!='e') {
      static const cpct_keyID keys[3] = {Key_A, Key_D, Key_W};
      k = compruebaTeclas(keys, 3);
   }
   else {
      if(e->forzarAndar==1)
      {
         k=comprobarSalto(e);
      }else
      {
         k=comprobarMovimientoEnemigo(e);
      }      
   }
   if(k == 2) {
         cambiarEstado(e, ST_prepSalto, 0);
      } else if(k == 1){
         cambiarEstado(e, ST_andando, M_derecha);
      } else if(k == 0){
         cambiarEstado(e, ST_andando, M_izquierda);
      }
}

void ascender(TEntidad* e) {
   desplazarEntidad(e, 0, e->pos.x[0], (e->pos.y[0]-cpct_get2Bits(g_tablaSalto, e->salto)));
   if (e->pos.y[0] < ORIGEN_MAPA_Y ) {
   		desplazarEntidad(e, 0, e->pos.x[0], ORIGEN_MAPA_Y);
         if((e->nbanana[0]!='e'))
   		    subirMapa();
   }
   if (++e->salto == PASOS_SALTO)
   	cambiarEstado(e, ST_cayendo, 0);
}

void saltando(TEntidad* e) {

   if ( e->nbanana[0]!='e') {
      if(!cpct_isKeyPressed(Key_W)){
         cambiarEstado(e, ST_cayendo, 0);
      }else {     

            static const cpct_keyID keys[2] = {Key_A, Key_D};
            u8 k;
               k = compruebaTeclas(keys, 2);
            if(k == 1){
               moverDerecha(e);
            } else if(k == 0){
               moverIzquierda(e);
            }
      }
   }else{
            u8 k;
               k = comprobarSalto(e);
            if(k == 1){
               moverDerecha(e);
            } else if(k == 0){
               moverIzquierda(e);
            }
      
   }
   if (esFondo(obtenerTilePtr(e->pos.x[0], e->pos.y[0]-1)) && esFondo(obtenerTilePtr(e->pos.x[0]+4, e->pos.y[0]-1))) {
            ascender(e);
         }
         else {
            cambiarEstado(e, ST_cayendo, 0);
         }
   
}

void prepSalto_animar(TEntidad* e) {
   cambiarEstado(e, ST_saltando, 0);
}

void prepSalto(TEntidad* e) {
   u8 k;
  
      static const cpct_keyID keys[3] = {Key_A,Key_D,Key_W};
       if(e->nbanana[0]!='e')
         {
            k = compruebaTeclas(keys, 3);
         }
         else{
            k = comprobarMovimientoEnemigo(e);
         }

      if(k == -1){
         cambiarEstado(e, ST_quieto, 0);
      } else if(k == 2){
         prepSalto_animar(e);
      }
}

void andando_animar(TEntidad* e, u8 mirar) {
	e->mira  = mirar;
	if(++e->nframe == ANDAR_FRAMES*ANIM_PAUSA)
      e->nframe = 0;
}

void andando(TEntidad* e) {
   u8 k;
   if(e->nbanana[0]!='e') {
      static const cpct_keyID keys[3] = {Key_A, Key_D, Key_W};
      k = compruebaTeclas(keys, 3);

   } else {
      if(e->forzarAndar==0){
       k = comprobarMovimientoEnemigo(e);
      }else{
         k=comprobarSalto(e);
      }
   }
   if(k == -1){
         cambiarEstado(e, ST_quieto, 0);
      } else if (k == 2){
         cambiarEstado(e, ST_prepSalto, 0);
      } else if (k == 1){
         moverDerecha(e);
         andando_animar(e, M_derecha);
      } else if (k == 0){
         moverIzquierda(e);
         andando_animar(e, M_izquierda);
      }
   
   if (!sobreSuelo(e)) {
      cambiarEstado(e, ST_cayendo, 0);
   }
   
   if(bordeInferior(e)) {
      muerteCaida(e);
   }
}

void ejecutarEstado(TEntidad* e) {

   if (e->estado == ST_quieto){
      quieto(e);
   } else if(e->estado == ST_andando) {
      andando(e);
   } else if(e->estado == ST_prepSalto) {
      prepSalto(e);
   } else if(e->estado == ST_saltando) {
      saltando(e);
   } else if(e->estado == ST_cayendo) {
      cayendo(e); 
   } else if(e->estado == ST_aterrizando) {
      aterrizando(e);
   }
   
   e->draw = 2;
   e->mover = SI;
}