#ifndef __ENTIDAD_H__
#define __ENTIDAD_H__

#include <cpctelera.h>
#include <types.h>

#define SI 1
#define NO 0
#define ANCHO_HERO      (12/2)
#define ALTO_HERO       19
#define HERO_FRAMES     16
#define ANIM_PAUSA      4
#define ANDAR_FRAMES    5
#define SALTAR_FRAMES   4
#define CAER_FRAMES     2
#define PASOS_SALTO     20

cpctm_declareMaskTable(g_tablatrans);    

typedef struct Pos {
   u8 x[3];
   u8 y[3];
   u8 num_mapa;
} TPos;

typedef struct Frame {
   u8  mira;
   u8* sprite;
} TFrame;

typedef struct Entidad {
	TPos pos;
   u8 w, h;
	u8 mover;
	u8 estado;
	const TFrame* frame;
	u8 nframe;
   u8 mira;
   u8 draw;
   u8 salto;
   u8 level;
   char nbanana[3];
   char nvida[2];
   u8 nodoActual;
   u8 nodoFuturo;
   u8 forzarAndar;
} TEntidad;

typedef enum {
      M_derecha   = 0
   ,  M_izquierda
} EMirar;

// Eventos del personaje
typedef enum {
      EV_detenerse = 0
   ,  EV_moverse
   ,  EV_saltar
   ,  EV_numEventos
} EEventos;

// Estados del personaje
typedef enum {
      ST_quieto  = 0
   ,  ST_andando
   ,  ST_prepSalto
   ,  ST_saltando
   ,  ST_cayendo
   ,  ST_aterrizando
   ,  ST_numEstados
} EEstados;

extern TEntidad entidad;

u8 compruebaTeclas(const cpct_keyID* k, u8 numk);
void inicializarEntidad(char u);
void asignarFrame(TEntidad* e, TFrame **animacion);
void girarFrame(TEntidad* e);
void seleccionarSpriteEntidad(TEntidad* e);
void dibujarEntidad(TEntidad* e);
void dibujarEntidad_initial(TEntidad* e);
void borrarEntidad(TEntidad* e);
void copy3(u8 *p);
void entityDrawnUpdate(TEntidad* e);
void desplazarEntidad(TEntidad* e, u8 i, u8 x, u8 y);
void ajustarAlSuelo(TEntidad* e);
void moverDerecha(TEntidad* e);
void moverIzquierda(TEntidad* e);
void cambiarEstado(TEntidad* e, u8 estado, u8 mira);
void aterrizando(TEntidad* e);
void descender(TEntidad* e);
void cayendo(TEntidad* e);
void quieto(TEntidad* e);
void ascender(TEntidad* e);
void saltando(TEntidad* e);
void prepSalto_animar(TEntidad* e);
void prepSalto(TEntidad* e);
void andando_animar(TEntidad* e, u8 mirar);
void andando(TEntidad* e);
void ejecutarEstado(TEntidad* e);

#endif